##########################################
# Convertor - Converts JSON to HTML page #
##########################################

import os
# for os.system
import math
from Utils import preDowloadAllAvatars, filenameFromUrl

# Not sure if these "constants" should be inside the class
# Everything that has "U" prefix is Unimplemented and will be ignored when passed
COMMENT_PAGING_STYLE_NONE = -1  # Nothing, just dumps everything to single html
# Links to another html file after every MAX_COMMENTS_PER_PAGE comment count
U_COMMENT_PAGING_STYLE_LINK = 0
# DEFAULT Loads more comments by embeding another html into current html
# once user scrolls enough - browser must support this
COMMENT_PAGING_STYLE_IFRAME = 1

# EXPERIMENTAL Only links avatars using their current link; not sure why would you wanted to do that
U_AVATAR_LINK_ONLY = 1
AVATAR_DOWNLOAD = 0  # DEFAULT - downloads avatars to their dir and links them with html
AVATAR_NONE = -1  # Will NOT download or link anything

# Contains only the stackable parts that actually link stuff
# if theres more needed (like for <video>) thats not inlcuded
_VIDEO_LINK_TEMPLATE = '<source src="{source}" type="{type}">\n'
_SCRIPT_LINK_TEMPLATE = '<script src="{source}"></script>\n'
_STYLE_LINK_TEMPLATE = '<link rel="stylesheet" type="text/css" href="{source}"/>\n'


# Class here works mostly as configurator
class HtmlConvertor:

    # Some global config
    # Dunno what "case" use for this as they are pretty much just config variables
    # So have Pascal_Snake
    Max_Comments_Per_Page = 500  # Todo: Test this, mainly with firefox
    Comment_Paging_Style = 1
    Avatar_Action = AVATAR_NONE

    # This is not exactly styled well Todo: Refactor options to single case and stick to it
    comments: list
    avatarDir: str
    videos: list
    styles: list
    scripts: list

    showProgress: bool = True  # Not usefull yet but in the future there may be longer operations

    # Constructor bude mít jen to bez čeho se absolutně nedá obejít, takže JSON a output file
    def __init__(self, comments):
        # Todo: Maybe add None check here?
        self.comments = comments

    def export(self, outputFile):
        """Exports the resulting html, applying properties of this class as modifications"""
        if self.Avatar_Action == AVATAR_DOWNLOAD:
            print("Downloading avatars")
            preDowloadAllAvatars(self.comments, self.avatarDir)

        print("Exporting to HTML...")
        writer = open(outputFile, "w")
        if (self.Max_Comments_Per_Page > 0):
            self.__writePaged(writer, outputFile)
        else:
            self.__writeMonolith(writer)
        writer.close()

    def __writePaged(self, mainWriter, mainName):
        """
        Writes as many HTMLs as needed using selected paging method

        [mainWriter]: Main file to be opened by user
        [mainName]: Name of that file, will be used for derivating the rest of filenames
        """
        print("Will be generating",
              math.ceil(len(self.comments) / self.Max_Comments_Per_Page), "additional HTML files")

        # First file for the user that will link all the others
        self.__writeHead(mainWriter)
        self.__writeVideoLink(mainWriter)

        totalPages = math.ceil(len(self.comments) / self.Max_Comments_Per_Page)
        # Put page links in <div> so they get rendered under video instead of next to it
        mainWriter.write('<div class="comment-link-box">\n')
        for i in range(1, totalPages + 1):  # Assuming the pages are generetaed by one less
            mainWriter.write('<a class="comment-iframe-switcher" href="#" onClick="switchCommentPage(\''
                             + str(i - 1) + mainName + '\')">' + str(i) + '</a>')

        mainWriter.write('\n</div>\n<div><iframe id="comment-iframe" src="0'
                         + mainName + '"></iframe></div>\n')
        self.__finalize(mainWriter)
        # Rest of the files
        self.__generateCommentPages(mainName)

    def __generateCommentPages(self, mainName):
        """
        Generates one html file for every Max_Comments_Per_Page number of comments
        Replies are not counted to this limit so the resulting number may be bigger
        """
        totalPages = (math.ceil(len(self.comments) / self.Max_Comments_Per_Page))

        for page in range(0, totalPages):
            # Total number of comments that are still neded to write
            remainingComments = len(self.comments) - (self.Max_Comments_Per_Page * page)
            alreadyWritten = len(self.comments) - remainingComments
            if (remainingComments >= self.Max_Comments_Per_Page):  # Ensure we write max allowed number of comments
                remainingComments = self.Max_Comments_Per_Page

            print("Writing page", page + 1, "/", totalPages)
            pageWriter = open(str(page) + mainName, "w")
            self.__writeHead(pageWriter)

            for c in range(0, remainingComments):
                self.__writeSingleComment(self.comments[alreadyWritten + c], pageWriter)
            self.__finalize(pageWriter)
            pageWriter.close()

    def __writeMonolith(self, writer):
        """Writes one big fat html with all comments in one file, it may take longer to load"""
        self.__writeHead(writer)
        self.__writeVideoLink(writer)
        self.__writeAllComments(writer)
        self.__finalize(writer)  # Writer closed in export

    # Writes head and links html with CSS
    def __writeHead(self, writer):
        """Writes HTML <Head> part with [writer]"""
        # Not sure if there is any better way of saving these strings but this looks awful
        # <!DOCTYPE html> is ommitted due to iframe size bug,
        # witht this declaration we are unable to size iframes properly
        headStart = """\
<html>
    <head>
        <meta charset="UTF-8">
        <title>{VideoName}</title>
"""
        headEnd = """\
    </head>
    <body onLoad="autoLoad()">
"""  # Todo: That autoload doesnt seem to work -> maybe because the script is included at the bottom?
        writer.write(headStart)
        try:
            for style in self.styles:
                writer.write(_STYLE_LINK_TEMPLATE.format(source=style))
        except NameError:
            print("No styles linked")
        writer.write(headEnd)

    # Change to general linking tht will link js, css, video
    def __writeVideoLink(self, writer):
        """Writes video html part to provided writer"""
        videoLinkOpen = "<video controls>\n"
        videoLinkClose = "\nYour browser does not support video playing.\n</video>"
        writer.write(videoLinkOpen)
        try:
            for videoFile in self.videos:
                writer.write(_VIDEO_LINK_TEMPLATE.format(source=videoFile,
                                                         type=self.__getVideoType(videoFile)))
        except NameError:
            print("No video files specified, not wiriting any videos")
        writer.write(videoLinkClose)

    def __writeAllComments(self, writer):
        """Decides which writing method to use and writes comments to html."""
        openComments = '<ul class="comment-secion">\n'
        writer.write(openComments)

        for com in self.comments:
            self.__writeSingleComment(com, writer)

        writer.write("</ul>\n")

    def __writeSingleComment(self, comment, writer):
        # We have to generate unique ID for reply ul that will be associated with parent
        writer.write('<li id="' + comment.getCid() + '">\n')
        if (self.Avatar_Action == AVATAR_DOWNLOAD):
            writer.write('<img src="' + self.avatarDir + filenameFromUrl(comment.photo)
                         + '.jpg" class="comment-avatar">\n')
        writer.write('<div class="comment-text-group">')
        writer.write('<p class="comment-author"><a href="' + comment.getChannelLink() + '" '
                     + 'target="_blank" rel="noopener noreferrer">'  # So we could open from iframe
                     + comment.author + '</a> (<em class="comment-author-cid">'
                     + comment.getCid() + '</em>)</p>\n')

        # Technically we dont need to write replying string as it should never be reply here
        writer.write('<p class="comment-text">' + comment.text + '</p>\n')
        if comment.hasReplies():  # We write all of it's children
            # Inject button here
            writer.write('<button class="show-reply-chain" type="button" onClick="toggleReplies(\''
                         + comment.getCid() + '\')">Toggle '
                         + str(len(comment.replies)) + ' replies</button>\n')

            # We have to close our grouping div before this <ul> but we cant close it here
            # because then it could go unclosed if there are no replies
            writer.write('</div>')
            writer.write('<ul class="comment-reply-chain" id="' + comment.getCid() + '-children">\n')
            for reply in comment.replies:  # Todo: We have to check for \ns so we have correct new lines<- ???
                writer.write('<li id="' + reply.getCid() + '" class="comment-reply">\n')
                # This has sto go inside div
                if (self.Avatar_Action == AVATAR_DOWNLOAD):
                    writer.write('<img src="' + self.avatarDir + filenameFromUrl(reply.photo)
                                 + '.jpg" class="comment-avatar">\n')
                writer.write('<div class="comment-text-group">\n')
                writer.write('<p class="comment-author"><a href="' + reply.getChannelLink() + '" '
                             + 'target="_blank" rel="noopener noreferrer">'  # So we could open from iframe
                             + reply.author + '</a>(<em class="comment-author-cid">'
                             + reply.getCid() + '</em>)</p>\n')
                writer.write('<p class="comment-quote"><a href=#' + reply.getParent()
                             + '> >> ' + reply.getParent() + '</a></p>\n')
                writer.write('<p class="comment-text">' + reply.text + '</p>\n')
                writer.write('</div>\n')
                writer.write('</li>\n')
            writer.write("</ul>\n")
        else:  # Closing grouping <div> if comments has NO replies
            writer.write('</div>')
        writer.write("</li>\n")

    def __finalize(self, writer):
        """Finishes writing html and links every JS file passed to this class"""
        try:
            for script in self.scripts:
                writer.write(_SCRIPT_LINK_TEMPLATE.format(source=script))
        except NameError:  # Should never happen, list should be at least empty
            print("No javascript linked")
        writer.write("</body>\n</html>\n")
        writer.flush()

    def __getVideoType(self, path):
        """Checks whether video type is supported and returns html indentifier of it's format"""
        if path.endswith("mp4") or path.endswith("ogg") or path.endswith("webm"):
            return "video/" + path[path.rfind(".") + 1: len(path)]
        else:
            # Todo: anything else we could do here? make this calling point to call another script from?
            # bridge/bridgefag
            # Well it looks it might doable in a single command or two.
            # This works in converting any video. It does NOT work actually carry the value over to the finished web page.
            # I tried several different alterations to the script but had no luck in the limited time I was able to test things well. 
            # Thus for now I only leave you with this for now.
            # Not sure if os.system is the way to go or if the editing should be done in this command, but if we are just going to call ffmpeg then it might be something to consider anyway.
            # Sorry for messy notations.
            print("Unsupported video: " + path + " Setting type to mp4")
            os.system("ffmpeg -i "  + path + " " + path.split("." , 1)[0] + ".mp4")
            return "video/mp4"
            # I guess the TD;LR is we need to figure out how to get return to give the altered path to return as well and be used in source in _VIDEO_LINK_TEMPLATE =
